import ij.*;// ImagePlus (entres autres)
import ij.gui.Overlay;
import ij.gui.Roi;
import ij.gui.PlotWindow;
import ij.process.ImageProcessor;
import ij.gui.Plot;
import ij.io.*;// FileInfo
import ij.measure.*;// Calibration (entres autres)
import ij.plugin.PlugIn;
import java.awt.*;


/**
 * @description
 * English description :
 * Plot the curve f( num slice, pixel value ( x_i, y_i) ) for each 
 *  ROI point selection (x_0, y0), (x_1, y_1), etc.
 * The curves colors turn from red to blue
 * The curve data are displayed in the log windows
 *
 * French description :
 * Trace la courbe f( num slice, valeur de pixel ( x_i, y_i) ) pour chaque 
 *  point du ROI (x_0, y0), (x_1, y_1), etc. (points simples ou multiples)
 * La couleur des courbes va du rouge au bleu
 * Les données des courbes est tracée dans le fichier .log aussi
 
 * @version v1.00
 * @author Vincent Vansuyt
 */
public class Plot_stacks_points implements PlugIn{
	
	/**
	 * Point d'entrée du plugin
	 * Trace la courbe des points pointés par le roi
	 * @param str 
	 */
	@Override
	public void run(String str) {
		// Vérification qu'il y a au moins une image ouverte
		ImagePlus imp;
		imp = WindowManager.getCurrentImage();
		if (imp == null) {
			IJ.noImage();
			return;
		}
		// Vérifications que l'image est en niveau de gris
		int nType = imp.getType();
		if ( nType != ImagePlus.GRAY8 &&
			 nType != ImagePlus.GRAY16 && 
			 nType != ImagePlus.GRAY32 ) {
			IJ.showMessage( "Warning", "Image must be gray scale");
			return;
		}
		// Calcul du niveau de gris maxi en fonction du type d'image
		int cl_Max = (int)Math.pow( 2.0 , 8.0 ) - 1;
		if ( nType == ImagePlus.GRAY16 ) {
			cl_Max = (int)Math.pow( 2.0 , 16.0 ) - 1;
		}
		else if( nType == ImagePlus.GRAY32 ) {
			cl_Max = (int)Math.pow( 2.0 , 32.0 ) - 1;
		}
		
		// - doit être stack
		int nNb_slices = imp.getNSlices();
		if ( nNb_slices <= 1 ) {
			IJ.showMessage("Warning", "Image must contain more than one stack");
			return;
		}
		
		// - doit avoir un ou plusieurs points dans son ROI
		Roi roi = imp.getRoi();
		if ( roi == null ) {
			IJ.showMessage("Warning", "Image must contain single or multi-points in the ROI");
			return;
		}
		Polygon poly;
		if ( roi.getType() == Roi.POINT || roi.getType() == Roi.POLYGON ) {
			poly = roi.getPolygon();
			IJ.log("  nb pts = " + Integer.toString( poly.npoints ));
		}
		else {
			IJ.showMessage("Warning", "ROI must contain single or multi-points");
			return;
		}
		
		// Parcours des stacks et collecte des points (tableaux d'entiers)
		ImageStack imgstack = imp.getImageStack();
		float[][] tabCl = new float[ poly.npoints ][ nNb_slices ];
		float[] tabAbscisses = new float[ nNb_slices ];
		for ( int s = 0; s < nNb_slices; ++s ) {
			tabAbscisses[ s ] = s+1;
			// Récupération de l'image du slice courant
			ImageProcessor img = imgstack.getProcessor( s+1 );
			// Parcours de la liste des points à collecter
			for ( int p = 0; p < poly.npoints; ++p ) {
				int x = poly.xpoints[ p ];
				int y = poly.ypoints[ p ];
				int cl_pix = img.getPixel( x, y );// (il est possible d'utiliser "float cl_pix = img.getPixelValue(s, s);")
				tabCl[ p ][ s ] = cl_pix;
			}
		}
		
		// Sortie du tableau de données dans le log
		IJ.log("===========================================================");
		// - calcul de la ligne "header"
		String strLigne = "num_slice";
		for ( int p = 0; p < poly.npoints; ++p ) {
			strLigne += " val_" + Integer.toString( p+1 );
		}
		IJ.log( strLigne );
		// - ajout des données
		for ( int s = 0; s < nNb_slices; ++s ) {
			strLigne = Integer.toString( (int)tabAbscisses[ s ] );
			for ( int p = 0; p < poly.npoints; ++p ) {
				strLigne += " ";
				strLigne += Integer.toString( (int)tabCl[p][s]);
			}
			IJ.log( strLigne );
		}
		IJ.log("===========================================================");

		PlotWindow.noGridLines = false; // draw grid lines
		Plot plot = new Plot("Pixel value","n° slice","color intensity", tabAbscisses, tabCl[0]);
		plot.setLimits(1, nNb_slices, 0, cl_Max);
		plot.setColor(Color.RED);
		plot.draw();
		// Ajout des autres courbes
		for ( int p = 1; p < poly.npoints; ++p ) {
			//plot.setLineWidth(2);
			//plot.addErrorBars(e);
			double i = (double)p * ( 255.0 / ((double)poly.npoints - 1 ) );
			int r = 255 - (int)Math.floor( i );
			int g = 0;
			int b = 255 - r;
			plot.setColor( new Color( r, g, b ));
			plot.addPoints( tabAbscisses, tabCl[p], PlotWindow.LINE);
		}
		plot.show();
	}// run
	
}// Plot_stacks_points
